'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);
var Tooltip = require('react-native-walkthrough-tooltip');
var Tooltip__default = _interopDefault(Tooltip);
var PropTypes = _interopDefault(require('prop-types'));
var events = require('events');

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;
  subClass.__proto__ = superClass;
}

var WAIT_NO_MORE_TIMEOUT = 1000 * 60 * 10; // 10 minutes

var HOT_SEC = 500;
var nullElement = {
  id: '',
  content: undefined,
  placement: undefined
};

var safeSetGuide = function safeSetGuide(element) {
  return {
    currentGuide: element
  };
};

var safeSetElement = function safeSetElement(element) {
  return {
    currentElement: element
  };
};

var WalkthroughContext = /*#__PURE__*/React__default.createContext({
  currentElement: nullElement,
  goToNext: function goToNext() {}
});

var ContextWrapper = /*#__PURE__*/function (_Component) {
  _inheritsLoose(ContextWrapper, _Component);

  function ContextWrapper(props) {
    var _this;

    _this = _Component.call(this, props) || this;

    _this.getCurrentElementIndex = function () {
      return _this.state.currentGuide.findIndex(function (element) {
        return element.id === _this.state.currentElement.id;
      });
    };

    _this.clearGuide = function () {
      return _this.setState(safeSetGuide([]));
    };

    _this.clearCurrentPossibleOutcomes = function () {
      var eventEmitter = _this.props.eventEmitter;

      _this.state.currentPossibleOutcomes.forEach(function (_ref) {
        var event = _ref.event,
            action = _ref.action;
        eventEmitter.removeListener(event, action);
      });

      _this.setState({
        currentPossibleOutcomes: [],
        outcomeListenerStartTimestamp: undefined
      });
    };

    _this.addTimeoutCheckToOutcomeActions = function (_ref2) {
      var event = _ref2.event,
          originalAction = _ref2.action;
      return {
        event: event,
        action: function action() {
          var outcomeListenerStartTimestamp = _this.state.outcomeListenerStartTimestamp;

          if (outcomeListenerStartTimestamp === undefined) {
            console.warn('[react-native-walkthrough] outcomeListenerStartTimestamp not initialized');
          } else if (Date.now() - outcomeListenerStartTimestamp >= WAIT_NO_MORE_TIMEOUT) {
            _this.clearGuide();
          } else {
            originalAction();
          }

          _this.clearCurrentPossibleOutcomes();
        }
      };
    };

    _this.listenForPossibleOutcomes = function (element) {
      var eventEmitter = _this.props.eventEmitter;
      var possibleOutcomes = element.possibleOutcomes;

      if (possibleOutcomes) {
        if (!Array.isArray(possibleOutcomes)) {
          console.warn('[react-native-walkthrough] non-Array value provided to possibleOutcomes');
        } else {
          _this.setState({
            currentPossibleOutcomes: possibleOutcomes.map(_this.addTimeoutCheckToOutcomeActions),
            outcomeListenerStartTimestamp: Date.now()
          }, function () {
            _this.state.currentPossibleOutcomes.forEach(function (_ref3) {
              var event = _ref3.event,
                  action = _ref3.action;
              return eventEmitter.once(event, action);
            });
          });
        }
      }
    };

    _this.setElementNull = function () {
      return _this.setState(safeSetElement(nullElement));
    };

    _this.setElement = function (element) {
      if (element.id !== _this.state.currentElement.id) {
        // clear previous element
        _this.setElementNull();

        _this.clearCurrentPossibleOutcomes(); // after a hot sec, set current element


        setTimeout(function () {
          _this.setState(safeSetElement(element));
        }, HOT_SEC);
      }
    };

    _this.setGuide = function (guide, callback) {
      _this.setElementNull();

      _this.setState(safeSetGuide(guide), callback);
    };

    _this.waitForTrigger = function (element, triggerEvent) {
      var eventEmitter = _this.props.eventEmitter;
      var waitStart = Date.now();
      var triggerGuide = JSON.stringify(_this.state.currentGuide);

      _this.setElementNull();

      eventEmitter.once(triggerEvent, function () {
        var waitEnd = Date.now();
        var currentGuide = JSON.stringify(_this.state.currentGuide);

        if (waitEnd - waitStart >= WAIT_NO_MORE_TIMEOUT) {
          _this.clearGuide();
        } else if (triggerGuide === currentGuide) {
          // only do action if we are still in the same guide that requested the element
          _this.setElement(element);
        }
      });
    };

    _this.goToElement = function (element) {
      if (element.triggerEvent) {
        _this.waitForTrigger(element, element.triggerEvent);
      } else {
        _this.setElement(element);
      }
    };

    _this.goToElementWithId = function (id) {
      var elementWithId = _this.state.currentGuide.find(function (element) {
        return element.id === id;
      });

      if (elementWithId) {
        _this.goToElement(elementWithId);
      }
    };

    _this.goToNext = function () {
      var currentElement = _this.state.currentElement;
      var nextIndex = _this.getCurrentElementIndex() + 1;

      if (currentElement.possibleOutcomes) {
        _this.listenForPossibleOutcomes(currentElement);

        _this.setElementNull();
      } else if (nextIndex < _this.state.currentGuide.length) {
        _this.goToElement(_this.state.currentGuide[nextIndex]);
      } else {
        _this.setElementNull();

        _this.clearGuide();
      }
    };

    _this.state = {
      currentElement: nullElement,
      currentGuide: [],
      currentPossibleOutcomes: [],
      outcomeListenerStartTimestamp: undefined
    };
    return _this;
  }

  var _proto = ContextWrapper.prototype;

  _proto.render = function render() {
    return React__default.createElement(WalkthroughContext.Provider, {
      value: _extends({}, this.state, {
        goToNext: this.goToNext
      })
    }, this.props.children);
  };

  return ContextWrapper;
}(React.Component);

var WalkthroughElement = function WalkthroughElement(props) {
  var elementId = props.id;
  var defaultPlacement = React__default.Children.count(props.children) === 0 ? 'center' : 'top';
  return React__default.createElement(WalkthroughContext.Consumer, null, function (_ref) {
    var currentElement = _ref.currentElement,
        goToNext = _ref.goToNext;
    var defaultTooltipProps = {
      useInteractionManager: true,
      isVisible: elementId === currentElement.id,
      content: props.content || currentElement.content,
      placement: currentElement.placement || defaultPlacement,
      onClose: function onClose() {
        var _currentElement$onClo;

        goToNext();
        (_currentElement$onClo = currentElement.onClose) === null || _currentElement$onClo === void 0 ? void 0 : _currentElement$onClo.call(currentElement);
      }
    };

    var tooltipProps = _extends({}, defaultTooltipProps, {}, currentElement.tooltipProps, {}, props.tooltipProps);

    return React__default.createElement(Tooltip__default, Object.assign({}, tooltipProps), props.useTooltipChildContext ? React__default.createElement(Tooltip.TooltipChildrenContext.Consumer, null, props.children) : props.children);
  });
};

WalkthroughElement.defaultProps = {
  content: undefined,
  tooltipProps: undefined,
  useTooltipChildContext: false
};

var wrapperRef = /*#__PURE__*/React__default.createRef();
var ee = /*#__PURE__*/new events.EventEmitter();

var WalkthroughProvider = function WalkthroughProvider(_ref) {
  var children = _ref.children;
  return React__default.createElement(ContextWrapper, {
    ref: wrapperRef,
    eventEmitter: ee
  }, children);
};

var goToWalkthroughElementWithId = function goToWalkthroughElementWithId(id) {
  var wrapper = wrapperRef.current;
  wrapper === null || wrapper === void 0 ? void 0 : wrapper.goToElementWithId(id);
};

var goToWalkthroughElement = function goToWalkthroughElement(element) {
  var wrapper = wrapperRef.current;
  wrapper === null || wrapper === void 0 ? void 0 : wrapper.goToElement(element);
};

var setWalkthroughGuide = function setWalkthroughGuide(guide, setGuide) {
  var wrapper = wrapperRef.current;
  wrapper === null || wrapper === void 0 ? void 0 : wrapper.setGuide(guide, setGuide);
};

var startWalkthrough = function startWalkthrough(walkthrough) {
  if (Array.isArray(walkthrough)) {
    setWalkthroughGuide(walkthrough, function () {
      if (walkthrough.length > 0) goToWalkthroughElement(walkthrough[0]);
    });
  } else {
    console.warn('[react-native-walkthrough] non-Array argument provided to startWalkthrough');
  }
};

var startWalkthroughAtElement = function startWalkthroughAtElement(walkthrough, elementId) {
  if (Array.isArray(walkthrough)) {
    setWalkthroughGuide(walkthrough, function () {
      goToWalkthroughElementWithId(elementId);
    });
  } else {
    console.warn('[react-native-walkthrough] non-Array argument provided to startWalkthroughAtElement');
  }
};

var dispatchWalkthroughEvent = function dispatchWalkthroughEvent(event) {
  return ee.emit(event);
};

var exitWalkthrough = function exitWalkthrough() {
  return startWalkthrough([nullElement]);
};

WalkthroughProvider.propTypes = {
  children: PropTypes.element
};

exports.WalkthroughContext = WalkthroughContext;
exports.WalkthroughElement = WalkthroughElement;
exports.WalkthroughProvider = WalkthroughProvider;
exports.dispatchWalkthroughEvent = dispatchWalkthroughEvent;
exports.exitWalkthrough = exitWalkthrough;
exports.goToWalkthroughElementWithId = goToWalkthroughElementWithId;
exports.nullElement = nullElement;
exports.startWalkthrough = startWalkthrough;
exports.startWalkthroughAtElement = startWalkthroughAtElement;
//# sourceMappingURL=react-native-walkthrough.cjs.development.js.map
